/**
 *  \file IMP/container/SingletonsRestraint.h
 *  \brief Apply a SingletonScore to each Singleton in a list.
 *
 *  WARNING This file was generated from SingletonsRestraint.h
 *  in /builddir/build/BUILD/imp-2.3.0/imp/tools/build/container_templates/container
 *  by tools/build/make_containers.py.
 *
 *  Copyright 2007-2014 IMP Inventors. All rights reserved.
 *
 */

#ifndef IMPCONTAINER_SINGLETONS_RESTRAINT_H
#define IMPCONTAINER_SINGLETONS_RESTRAINT_H

#include <IMP/container/container_config.h>
#include <IMP/kernel/internal/ContainerRestraint.h>
#include <IMP/kernel/SingletonContainer.h>
#include <IMP/kernel/SingletonScore.h>

IMPCONTAINER_BEGIN_NAMESPACE

//! Applies a SingletonScore to each Singleton in a list.
/** This restraint stores the used particles in a kernel::ParticlesTemp.
    The container used can be set so that the list can be shared
    with other containers (or a nonbonded list can be used).

    Examples using various multiplicity containers:
    \include restrain_in_sphere.py
    \include nonbonded_interactions.py

    \see IMP::core::SingletonRestraint
 */
class SingletonsRestraint :
#if defined(SWIG) || defined(IMP_DOXYGEN)
    public kernel::Restraint
#else
    public IMP::kernel::internal::ContainerRestraint<kernel::SingletonScore,
                                                     kernel::SingletonContainer>
#endif
    {
  typedef IMP::kernel::internal::ContainerRestraint<
      kernel::SingletonScore, kernel::SingletonContainer> P;

 public:
  //! Create the restraint with a shared container
  /** \param[in] ss The function to apply to each particle.
      \param[in] pc The container containing the stored particles. This
      container is not copied.
      \param[in] name The object name
   */
  SingletonsRestraint(kernel::SingletonScore *ss,
                      kernel::SingletonContainerAdaptor pc,
                      std::string name = "SingletonsRestraint %1%")
      : P(ss, pc, name) {}

#if defined(IMP_DOXYGEN) || defined(SWIG)
  double unprotected_evaluate(IMP::kernel::DerivativeAccumulator *accum) const;
  IMP::kernel::ModelObjectsTemp do_get_inputs() const;
  IMP_OBJECT_METHODS(SingletonsRestraint)
#endif
};

IMP_OBJECTS(SingletonsRestraint, SingletonsRestraints);

IMPCONTAINER_END_NAMESPACE

#endif /* IMPCONTAINER_SINGLETONS_RESTRAINT_H */
