/**
 *  \file IMP/container/TripletsConstraint.h
 *  \brief Use a TripletModifier applied to a kernel::ParticleTripletsTemp to
 *  maintain an invariant
 *
 *  WARNING This file was generated from TripletsConstraint.h
 *  in /builddir/build/BUILD/imp-2.3.0/imp/tools/build/container_templates/container
 *  by tools/build/make_containers.py.
 *
 *  Copyright 2007-2014 IMP Inventors. All rights reserved.
 */

#ifndef IMPCONTAINER_TRIPLETS_CONSTRAINT_H
#define IMPCONTAINER_TRIPLETS_CONSTRAINT_H

#include <IMP/container/container_config.h>
#include <IMP/TripletContainer.h>
#include <IMP/TripletModifier.h>
#include <IMP/Constraint.h>
#include <IMP/base/object_macros.h>
#include <IMP/kernel/internal/ContainerConstraint.h>

IMPKERNEL_BEGIN_NAMESPACE
// for swig
class TripletContainer;
class TripletModifier;
IMPKERNEL_END_NAMESPACE

IMPCONTAINER_BEGIN_NAMESPACE
//! Apply a TripletFunction to a TripletContainer to maintain an invariant
/** The score state is passed up to two TripletModifiers, one to
    apply before evaluation and the other after. The one after
    should take a DerivativeAccumulator as its last argument for
    TripletModifier::apply() and will only be called if
    the score was computed with derivatives.

    \see core::TripletConstraint
 */
class TripletsConstraint :
#if defined(SWIG) || defined(IMP_DOXYGEN)
    public Constraint
#else
    public IMP::kernel::internal::ContainerConstraint<
        TripletModifier, TripletModifier, TripletContainer>
#endif
    {
  typedef IMP::kernel::internal::ContainerConstraint<
      TripletModifier, TripletModifier, TripletContainer> P;

 public:
  /** \param[in] c The Container to hold the elements to process
      \param[in] before The TripletModifier to apply to all elements
      before evaluate.
      \param[in] after The TripletModifier to apply to all elements
      after evaluate.
      \param[in] name The object name
   */
  TripletsConstraint(TripletModifier *before, TripletModifier *after,
                       TripletContainerAdaptor c,
                       std::string name = "TripletConstraint %1%")
      : P(before, after, c, name) {}
#if defined(IMP_DOXYGEN) || defined(SWIG)
 protected:
  void do_update_attributes();
  void do_update_derivatives(DerivativeAccumulator *da);
  virtual kernel::ModelObjectsTemp do_get_inputs() const;
  virtual kernel::ModelObjectsTemp do_get_outputs() const;
  IMP_OBJECT_METHODS(TripletsConstraint);
#endif
};

IMP_OBJECTS(TripletsConstraint, TripletsConstraints);

IMPCONTAINER_END_NAMESPACE

#endif /* IMPCONTAINER_TRIPLETS_CONSTRAINT_H */
