/**
 *  \file IMP/kernel/PairContainer.h
 *  \brief A container for Pairs.
 *
 *  WARNING This file was generated from PairContainer.h
 *  in /builddir/build/BUILD/imp-2.3.0/imp/tools/build/container_templates/kernel
 *  by tools/build/make_containers.py.
 *
 *  Copyright 2007-2014 IMP Inventors. All rights reserved.
 */

#ifndef IMPKERNEL_PAIR_CONTAINER_H
#define IMPKERNEL_PAIR_CONTAINER_H

#include <IMP/kernel/kernel_config.h>
#include "internal/IndexingIterator.h"
#include "Particle.h"
#include "container_base.h"
#include "internal/container_helpers.h"
#include "DerivativeAccumulator.h"
#include "ParticleTuple.h"
#include <IMP/base/ref_counted_macros.h>
#include <IMP/base/check_macros.h>
#include <IMP/base/Pointer.h>
#include <IMP/base/InputAdaptor.h>
#include <IMP/base/utility_macros.h>
#include <IMP/base/deprecation_macros.h>
#include <algorithm>

IMPKERNEL_BEGIN_NAMESPACE
class PairModifier;
class PairScore;

//! A shared container for Pairs
/** Stores a shared collection of Pairs.
 */
class IMPKERNELEXPORT PairContainer : public Container {
 public:
  typedef kernel::ParticlePair ContainedType;
  typedef kernel::ParticlePairsTemp ContainedTypes;
  typedef kernel::ParticleIndexPairs ContainedIndexTypes;
  typedef kernel::ParticleIndexPair ContainedIndexType;
  typedef PairModifier Modifier;
  typedef const kernel::ParticleIndexPair& PassContainedIndexType;

  //! Just use apply() in the base class
  void apply_generic(const PairModifier *m) const;

  //! Apply a SingletonModifier to the contents
  void apply(const PairModifier *sm) const;

  /** Get all the indexes that might possibly be contained in the
      container, useful with dynamic containers. For example,
      with a container::ClosePairContainer, this is the list
      of all pairs taken from input list (those that are far apart
      in addition to those that are close).
  */
  virtual kernel::ParticleIndexPairs get_range_indexes() const = 0;

  const kernel::ParticleIndexPairs &get_contents() const {
    if (get_provides_access())
      return get_access();
    else {
      std::size_t nhash = get_contents_hash();
      if (contents_hash_ != nhash || !cache_initialized_) {
        contents_hash_ = nhash;
        cache_initialized_ = true;
        contents_cache_ = get_indexes();
      }
      return contents_cache_;
    }
  }

  /** Get all the indexes contained in the container.

    This should be protected but isn't for compatibility reasons.

    External callers should use get_contents().
  */
  virtual kernel::ParticleIndexPairs get_indexes() const = 0;

#ifndef IMP_DOXYGEN

  kernel::ParticlePairsTemp get() const {
    return IMP::kernel::internal::get_particle(get_model(), get_indexes());
  }

  kernel::ParticlePair get(unsigned int i) const {
    return IMP::kernel::internal::get_particle(get_model(), get_indexes()[i]);
  }
  unsigned int get_number() const { return get_indexes().size(); }
#ifndef SWIG
  bool get_provides_access() const;
  virtual const kernel::ParticleIndexPairs &get_access() const {
    IMP_THROW("Object not implemented properly.", base::IndexException);
  }

  template <class Functor>
  Functor for_each(Functor f) {
    kernel::ParticleIndexPairs vs = get_indexes();
    // use boost range instead
    return std::for_each(vs.begin(), vs.end(), f);
  }

#endif
#endif

  /** \deprecated_at{2.1} Use get_indexes() instead.
   */
  kernel::ParticlePairsTemp get_particle_pairs() const;

  /** \deprecated_at{2.1} This can be very slow and is probably not useful
   */
  IMPKERNEL_DEPRECATED_METHOD_DECL(2.1)
  unsigned int get_number_of_particle_pairs() const;

  /** \deprecated_at{2.1}Use get_indexes() instead and thing about using the
      IMP_CONTAINER_FOREACH() macro.*/
  IMPKERNEL_DEPRECATED_METHOD_DECL(2.1)
  kernel::ParticlePair get_particle_pair(unsigned int i) const;

 protected:
  PairContainer(kernel::Model *m,
                     std::string name = "PairContainer %1%");

  virtual void do_apply(const PairModifier *sm) const = 0;
  virtual bool do_get_provides_access() const { return false; }

  IMP_REF_COUNTED_NONTRIVIAL_DESTRUCTOR(PairContainer);

 private:
  mutable std::size_t contents_hash_;
  mutable kernel::ParticleIndexPairs contents_cache_;
  mutable bool cache_initialized_;
};

/** This class allows either a list or a container to be
    accepted as input.
*/
class IMPKERNELEXPORT PairContainerAdaptor :
#if !defined(SWIG) && !defined(IMP_DOXYGEN)
    public base::Pointer<PairContainer>
#else
    public base::InputAdaptor
#endif
    {
  typedef base::Pointer<PairContainer> P;

 public:
  PairContainerAdaptor() {}

  /**
     Constructs the adaptor pointing to c (so if the contents of c are changed
     dynamically, so do the contents of the adaptor, and vice versa)
   */
  PairContainerAdaptor(PairContainer *c);

  /**
     Constructs the adaptor pointing to c (so if the contents of c are changed
     dynamically, so do the contents of the adaptor, and vice versa)
   */
  template <class C>
  PairContainerAdaptor(base::internal::PointerBase<C> c)
      : P(c) {}

  /**
     Adapts the non-empty list t to PairContainer

     @param t a non-empty list of kernel::ParticlePairsTemp
  */
  PairContainerAdaptor(const kernel::ParticlePairsTemp &t);

  /** Set the name of the resulting container if it is currently the
      default value. */
  void set_name_if_default(std::string name);
};

IMPKERNEL_END_NAMESPACE

#endif /* IMPKERNEL_PAIR_CONTAINER_H */
