/**
 *  \file IMP/kernel/pair_macros.h
 *  \brief Macros for various classes.
 *
 *  This file is generated by a script (core/tools/make-containers).
 *  Do not edit directly.
 *
 *  Copyright 2007-2014 IMP Inventors. All rights reserved.
 */

#ifndef IMPKERNEL_PAIR_MACROS_H
#define IMPKERNEL_PAIR_MACROS_H

#include "internal/TupleRestraint.h"
#include "internal/functors.h"
#include "container_macros.h"
#include <IMP/base/object_macros.h>
#include <algorithm>

/** Define
    - IMP::kernel::PairScore::evaluate_indexes()
    - IMP::kernel::PairScore::evaluate_if_good_indexes()
 */
#define IMP_PAIR_SCORE_METHODS(Name)                                      \
  double evaluate_indexes(kernel::Model *m, const kernel::ParticleIndexPairs &p,          \
                          DerivativeAccumulator *da, unsigned int lower_bound, \
                          unsigned int upper_bound) const IMP_FINAL {          \
    double ret = 0;                                                            \
    for (unsigned int i = lower_bound; i < upper_bound; ++i) {                 \
      ret += evaluate_index(m, p[i], da);                                      \
    }                                                                          \
    return ret;                                                                \
  }                                                                            \
  double evaluate_if_good_indexes(                                             \
      kernel::Model *m, const kernel::ParticleIndexPairs &p, DerivativeAccumulator *da,   \
      double max, unsigned int lower_bound, unsigned int upper_bound) const {  \
    double ret = 0;                                                            \
    for (unsigned int i = lower_bound; i < upper_bound; ++i) {                 \
      ret += evaluate_if_good_index(m, p[i], da, max - ret);                   \
      if (ret > max) return std::numeric_limits<double>::max();                \
    }                                                                          \
    return ret;                                                                \
  }

//! Define extra the functions needed for a PairPredicate
#define IMP_PAIR_PREDICATE_METHODS(Name)                                  \
  int get_value(const kernel::ParticlePair& a) const {                                        \
    return get_value_index(IMP::kernel::internal::get_model(a),                \
                           IMP::kernel::internal::get_index(a));               \
  }                                                                            \
  Ints get_value(const kernel::ParticlePairsTemp &o) const {                          \
    Ints ret(o.size());                                                        \
    for (unsigned int i = 0; i < o.size(); ++i) {                              \
      ret[i] += Name::get_value(o[i]);                                         \
    }                                                                          \
    return ret;                                                                \
  }                                                                            \
  Ints get_value_index(kernel::Model *m, const kernel::ParticleIndexPairs &o) const {     \
    Ints ret(o.size());                                                        \
    for (unsigned int i = 0; i < o.size(); ++i) {                              \
      ret[i] += Name::get_value_index(m, o[i]);                                \
    }                                                                          \
    return ret;                                                                \
  }                                                                            \
  IMP_IMPLEMENT_INLINE_NO_SWIG(                                                \
      void remove_if_equal(kernel::Model *m, kernel::ParticleIndexPairs &ps,              \
                           int value) const,                                   \
  {                                                                            \
        ps.erase(                                                              \
            std::remove_if(ps.begin(), ps.end(),                               \
                           IMP::kernel::internal::PredicateEquals<Name, true>( \
                               this, m, value)),                               \
            ps.end());                                                         \
      });                                                                      \
  IMP_IMPLEMENT_INLINE_NO_SWIG(void remove_if_not_equal(kernel::Model *m,      \
                                                        kernel::ParticleIndexPairs &ps,   \
                                                        int value) const,      \
  {                                                                            \
    ps.erase(                                                                  \
        std::remove_if(ps.begin(), ps.end(),                                   \
                       IMP::kernel::internal::PredicateEquals<Name, false>(    \
                           this, m, value)),                                   \
        ps.end());                                                             \
  });

//! Use IMP_PAIR_MODIFIER() instead
#define IMP_PAIR_DERIVATIVE_MODIFIER(Name) IMP_PAIR_MODIFIER(Name)

/** Define
    - IMP::kernel::PairModifier::apply_indexes()
*/
#define IMP_PAIR_MODIFIER_METHODS(Name)                             \
  virtual void apply_indexes(kernel::Model *m, const kernel::ParticleIndexPairs &o, \
                             unsigned int lower_bound,                   \
                             unsigned int upper_bound) const IMP_FINAL { \
    for (unsigned int i = lower_bound; i < upper_bound; ++i) {           \
      apply_index(m, o[i]);                                              \
    }                                                                    \
  }

//! Use IMP_INDEX_PAIR_MODIFIER instead
#define IMP_INDEX_PAIR_DERIVATIVE_MODIFIER(Name) \
  IMP_INDEX_PAIR_MODIFIER(Name)

#ifndef IMP_DOXYGEN
#define IMP_IMPLEMENT_PAIR_CONTAINER(Name)                          \
  IMP_IMPLEMENT_INLINE(void do_apply(const PairModifier *sm) const, \
  { apply_generic(sm); });                                               \
  virtual ParticleIndexes get_all_possible_indexes() const IMP_OVERRIDE; \
  IMP_OBJECT_NO_WARNING(Name)
#endif

/** Use this to fill in container methods
    IMP::kernel::PairContainer::do_apply()
*/
#define IMP_PAIR_CONTAINER_METHODS(Name) \
  void do_apply(const PairModifier *sm) const { apply_generic(sm); }

#endif /* IMPKERNEL_PAIR_MACROS_H */
